/* brief program that demonstrates .TraceObject and tests intercepting traceObjects,
   while running the supplied program repeatedly each time with a different .TraceObject
   option
*  2024-01-28, rgf
*/
parse arg fn
if \SysFileExists(fn) then
do
   say "file ["fn"] does not exist, aborting ..."
   exit -1
end

do counter c option over "N", "T", "S", "F", "P"   -- Normal, Thread, Standard, Full, Profiling
   say "RUN #" c": option='"option"' ..." "(so far there were" .traceObject~counter "TraceObjects created)"
   arr=.array~new
   .TraceObject~option=option       -- global
   .TraceObject~collector=arr       -- global
say "... about    to   CALL" fn "...." "calling  in RUN #" c
   call runner fn, option, arr
say "... RETURNED from CALL" fn "...." "returned in RUN #" c "(this run created another" arr~items "TraceObjects)"
/* --
do counter c3 e over arr
   say "/// #" c3~right(3)": e~class~id="e~class~id "e={"e"}"
end
-- */
   say "-"~copies(79)
   say
end


::routine runner     -- in case the called program has an error, intercept it here
   use arg fn, option, arr
   signal on syntax
   say
   say "about to call ["fn"] with option='"option"' arr~items="arr~items "..."
   call (fn)

syntax:
say "#" .line "..."
   if VAR("SIGL") then  -- a condition occurred?
   do
      co=condition('o')
      say "SIGNAL   from calling ["fn"] with option="option "arr~items="arr~items "..."
      say "        name:" co~name":" co~instruction "sigl:" sigl
      say "   errortext:" co~errortext
      say "     message:" co~message
      say
   end
   else
      say "returned from calling ["fn"] with option='"option"' arr~items="arr~items "..."
   say ".TraceObject~option:" .TraceObject~option ".TraceObject~collector:" .TraceObject~collector~objectname
   say
   say "waiting a little bit to have all threads exited (they generate additional traceObjects) ..."
   call syssleep .1
   .TraceObject~collector=.nil      -- now stop collecting
   say center(" option '"option"': arr~items:" arr~items "",69,"-")

