/* brief program that demonstrates .TraceObject and intercepting traceObjects
   supplying feedback and running the supplied program repeatedly each time
   with a different .TraceObject option
*  2024-01-28, rgf
*/
parse arg fn
if \SysFileExists(fn) then
do
   say "file ["fn"] does not exist, aborting ..."
   exit -1
end

oldDest=.traceOutput~current
say "oldDest:" oldDest~class"/"oldDest~objectName"/"oldDest~identityHash"/"oldDest

do counter c option over "N", "T", "S", "F", "P"   -- Normal, Thread, Standard, Full, Profiling
   say "RUN #" c": option='"option"' ..." "(so far there were" .traceObject~counter "TraceObjects created)"
   arr=.array~new
   .TraceObject~option=option         -- global setting
   .TraceObject~collector=arr       -- global setting
   i=.interceptor~new(oldDest)
   say "i       :" i~identityHash"/"i
   .traceOutput~destination(i)      -- push i
    currDest=.traceOutput~current
   say "currDest:" currDest~identityHash"/"currDest

say "... about    to   CALL" fn "...." "calling  in RUN #" c
   call runner fn, option, arr
say "... RETURNED from CALL" fn "...." "returned in RUN #" c  "(this run created another" arr~items "TraceObjects)"

   .TraceObject~collector=.nil
   say center(option":" i~counter "TraceObjects intercepted, arr~items:" arr~items,69,"-")
   .traceOutput~destination          -- pop i
   say "-"~left(79)
   say
end

  -- demonstrate how to code an interceptor to do what the Rexx programmer wishes to do
::class interceptor
::method init
  expose counter oldDest
  use strict arg oldDest   -- i.e. .traceOutput~current before adding the interceptor
  counter=0

::attribute oldDest  -- old destination object
::attribute counter

::method lineout
  expose counter oldDest
  use arg traceObject
   -- may change the traceLine, then the changed traceLine needs to get forwarded
  counter+=1
      -- output directrly to stderr for debugging purposes:
  .stderr~say("... LINEOUT: counter="counter "| traceObject={"traceObject"} ...")
  forward to (oldDest)  -- just a demo, forward traceObject unchanged

::method unknown     -- make sure unknown messages get forwarded as well
  expose oldDest
  use arg msgName, msgArgs
      -- output directrly to stderr for debugging purposes:
  .stderr~say("... UNKNOWN: msgName="msgName "| msgArgs={"msgArgs~makeString('L',',')"} ...")
  forward to (oldDest) message (msgName) arguments (msgArgs)



::routine runner     -- in case the called program has an error, intercept it here
   use arg fn, option, arr
   signal on syntax
   say
   say
   say "---> "~copies(10)
   say "about to call ["fn"] with option='"option"' arr~items="arr~items "..."
   call (fn)

syntax:
   if VAR("SIGL") then  -- a condition occurred?
   do
      co=condition('o')
      say "SIGNAL   from calling ["fn"] with option="option "arr~items="arr~items "..."
      say "        name:" co~name":" co~instruction "sigl:" sigl
      say "   errortext:" co~errortext
      say "     message:" co~message
      say
   end
   else
      say "returned from calling ["fn"] with option='"option"' arr~items="arr~items "..."
   say "<--- "~copies(10)
   say
   say ".TraceObject~option:" .TraceObject~option ".TraceObject~collector:" .TraceObject~collector~objectname
   say
   say "waiting a little bit to have all threads exited (they generate additional traceObjects) ..."
   call syssleep .1
   .TraceObject~collector=.nil
   say center(" option '"option"': arr~items:" arr~items "",69,"-")

