/* brief program that demonstrates .TraceObject and intercepting traceObjects
   supplying feedback and running the supplied program repeatedly each time
   with a different .TraceObject option
*  2024-01-28, rgf
*/
parse arg fn
if \SysFileExists(fn) then
do
   say "file ["fn"] does not exist, aborting ..."
   exit -1
end

do counter c arrItem over ( ("floating method ...","myMakeString01", .methods  ~myMakeString01), -
                            ("resource ..."       ,"myMakeString02", .resources~myMakeString02), -
                            ("floating method (MFC) ...","myMakeString03", .methods  ~myMakeString03) )
   say "RUN #" c": using" arrItem[2] arrItem[1]
   .TraceObject~unsetMakeString     -- testing removing a set method
   .TraceObject~setMakeString(arrItem[3])

   arr=.array~new
   .TraceObject~collector=arr       -- global setting
say "... about    to   CALL" fn "...." "calling  in RUN #" c
   call runner fn, arr
say "... RETURNED from CALL" fn "...." "returned in RUN #" c  "(this run created another" arr~items "TraceObjects)"

   .TraceObject~collector=.nil
   say "-"~left(79)
   say
end

say "Testing unsetMakeString:"
say
.TraceObject~unsetMakeString     -- cancel any setMakeString invocation
.TraceObject~option="F"
arr=.array~new
.TraceObject~collector=arr       -- global setting
say "... about    to   CALL" fn "...." "calling  after all RUNs"
call runner fn, arr
say "... RETURNED from CALL" fn "...." "returned from all RUNs (this run created another" arr~items "TraceObjects)"
say "-"~left(79)


/* This is a "floating" method, i.e. a method defined before the very first class directive
 * if any. Floating methods get collected in a directory named .methods in this package (program).
 *
 * One can access all entries in the supplied traceObject and create any string value
 * from it that the Rexx programmer wants.
 *
 * @param traceObj an instance of .TraceObject
 * @return the produced string rendering
*/
::method myMakeString01
  use arg traceObj
  return "Via floating method: ["traceObj~timeStamp~longTime"]" traceObj~traceline


/* This is the method's code, saved as a resource in this package (program).

a method, i.e. a method defined before the very first class directive
 * if any.
 *
 * One can access all entries in the supplied traceObject and create any string value
 * from it that the Rexx programmer wants.
 *
 * @param traceObj an instance of .TraceObject
 * @return the produced string rendering
*/
::resource myMakeString02
  use arg traceObj
  return "Via resource: ["traceObj~timeStamp~longTime"]" traceObj~traceline
::END


/* MFC's format suggestion, cf. <https://sourceforge.net/p/oorexx/mailman/oorexx-devel/thread/69B32C0CD99749A59516F36250A992E4%40MFC5/#msg37777673>
*/
::method myMakeString03
  use arg traceObj
  parse value traceObj~traceline with line 7 prefix rest
  prefixT = prefix~left(2) || traceObj["THREAD"] || prefix~right(1)
  return line prefixT rest


::routine runner     -- in case the called program has an error, intercept it here
   use arg fn, arr
   signal on syntax
   say
   say
   say "---> "~copies(10)
   call (fn)

syntax:
   if VAR("SIGL") then  -- a condition occurred?
   do
      co=condition('o')
      say "SIGNAL   from calling ["fn"] with option="option "arr~items="arr~items "..."
      say "        name:" co~name":" co~instruction "sigl:" sigl
      say "   errortext:" co~errortext
      say "     message:" co~message
      say
   end
   else
      say "returned from calling ["fn"] with option='"option"' arr~items="arr~items "..."
   say "<--- "~copies(10)
   say
   say "waiting a little bit to have all threads exited (they generate additional traceObjects) ..."
   call syssleep .1
   .TraceObject~collector=.nil
   say center("  arr~items:" arr~items "",69,"-")
