/* brief program that demonstrates .TraceObject and intercepting traceObjects
   defining one own's prefix depending on .TraceObject's option setting
   and running the supplied program repeatedly each time with a different
   .TraceObject option
   - demonstrate how to sort differently
   - demonstrate F and S difference at the end

*  2024-01-28/29, rgf
*/
signal on syntax
fn=.sysCargs[1]      -- fetch the file name
if \SysFileExists(fn) then
do
   say "file ["fn"] does not exist, aborting ..."
   exit -1
end

args=.nil
if .sysCargs~items>1 then
   args=.sysCargs~section(2)  -- assign remainders as arguments

oldDest=.traceOutput~current
say "oldDest:" oldDest~class"/"oldDest~objectName"/"oldDest~identityHash"/"oldDest

do counter c option over "N", "T", "S", "F", "P"   -- Normal, Thread, Standard, Full, Profiling
   say "RUN #" c": option='"option"' ..."
   arr=.array~new
   .TraceObject~option=option       -- global setting
   .TraceObject~collector=arr       -- global setting
   i=.interceptor~new(oldDest)
   say "i       :" i~identityHash"/"i
   .traceOutput~destination(i)      -- push i
    currDest=.traceOutput~current
   say "currDest:" currDest~identityHash"/"currDest

say "... about    to   CALL" fn "...." "calling  in RUN #" c
   call runner fn, args, option, arr
say "... RETURNED from CALL" fn "...." "returned in RUN #" c

   .TraceObject~collector=.nil
   say center(option":" i~counter "TraceObjects intercepted, arr~items:" arr~items,69,"-")
   .traceOutput~destination          -- pop i
   say "-"~left(79)
   say
end

   -- demonstrate sorting array, use F trace formattign
.traceObject~option='F' -- have makeString do a full prefix
say
say "==/*\== "~copies(10)
say "==\*/== "~copies(10)
say
tab="09"x   -- tabulator character
say "sorting traceObjects by thread, invocation, nr:"
say
myComparator=.sortByThreadInvocationNr~new
arr=arr~sortWith(myComparator)
do counter c traceObj over arr
   say tab adjRight(c,3)":" myTraceFormatting(traceObj)
end
say
say "==/*\== "~copies(10)
say "==\*/== "~copies(10)

say
say "sorting in the default order (by nr):"
say
arr~sort    -- sorts by NR ascendingly
do counter c traceObj over arr
   say tab adjRight(c,3)":" myTraceFormatting(traceObj)
end
say

say "==/*\== "~copies(10)
say "==\*/== "~copies(10)
say
say ".TraceObject~option:" .traceObject~option "..."
say
say "sorting traceObjects by invocation, nr:"
say
myComparator=.sortByInvocationNr~new
arr=arr~sortWith(myComparator)
do counter round opt over 'F', 'S', 'N'
   say
   say "-"~copies(50)
   say
   .TraceObject~option=opt
   say "ROUND #" round": now .TraceObject~option set to: '".traceObject~option"' (sorted by invocation, nr)"
   say
   say "iterating over collected TraceObjects:"
   say
   do counter c traceObj over arr
      say tab adjRight(c,3)":" myTraceFormatting(traceObj)
   end
   say
end
say

say "done. (A total of" .traceObject~counter "traceObjects got created.)"
exit


syntax:
      co=condition('o')
      say "SIGNAL   from calling ["fn"] with option="option "arr~items="arr~items "..."
      say "        name:" co~name":" co~instruction "sigl:" sigl
      say "   errortext:" co~errortext
      say "     message:" co~message
      say

::requires "traceutil.cls"

  -- demonstrate how to code an interceptor to do what the Rexx programmer wishes to do
::class interceptor
::method init
  expose counter oldDest
  use strict arg oldDest   -- i.e. .traceOutput~current before adding the interceptor
  counter=0

::attribute oldDest  -- old destination object
::attribute counter

::method lineout
  expose counter oldDest
  use arg traceObject
  signal on syntax   -- important if errors occur in this method

   -- may change the traceLine, then the changed traceLine needs to get forwarded
  counter+=1
  option=.traceObject~option
  if option='P' then return   -- we do not process profiling messages
  newTraceLine="(#"adjRight(traceObject~nr,4)"," traceObject~timeStamp")" traceObject~makeString
  select case option
     when 'N' then newTraceLine="<Normal!>  " newTraceLine
     when 'S' then newTraceLine="<Standard!>" newTraceLine
     when 'F' then newTraceLine="<Full!>    " newTraceLine
     otherwise nop
  end
  forward to (oldDest) array (newTraceLine) continue
  return

syntax:
   co=condition('o')
   say .context~name": OOPS!" "line #:" .line
   say "        name:" co~name":" co~instruction "sigl:" sigl
   say "   errortext:" co~errortext
   say "     message:" co~message


::method unknown     -- make sure unknown messages get forwarded as well
  expose oldDest
  use arg msgName, msgArgs
      -- output directrly to stderr for debugging purposes:
  .stderr~say("... UNKNOWN: msgName="msgName "| msgArgs={"msgArgs~makeString('L',',')"} ...")
  forward to (oldDest) message (msgName) arguments (msgArgs)



::routine runner     -- in case the called program has an error, intercept it here
   use arg fn, args, option, arr
   signal on syntax
   say
   say
   say "---> "~copies(10)
   say "about to call ["fn"] with option='"option"' arr~items="arr~items "..."
   r=.routine~newFile(fn)
   if args~isNil then r~call           -- run without arguments
                 else r~callWith(args) -- supply arguments for program

syntax:
   if VAR("SIGL") then  -- a condition occurred?
   do
      co=condition('o')
      say "SIGNAL   from calling ["fn"] with option="option "arr~items="arr~items "..."
      say "        name:" co~name":" co~instruction "sigl:" sigl
      say "   errortext:" co~errortext
      say "     message:" co~message
      say
   end
   else
      say "returned from calling ["fn"] with option='"option"' arr~items="arr~items "..."
   say "<--- "~copies(10)
   say
   say ".TraceObject~option:" .TraceObject~option ".TraceObject~collector:" .TraceObject~collector~objectname
   say
   say "waiting a little bit to have all threads exited (they generate additional traceObjects) ..."
   call syssleep .1
   .TraceObject~collector=.nil
   say center(" option '"option"': arr~items:" arr~items "",69,"-")
   return


::routine myTraceFormatting   -- prepend a prefix that shows NR and DATETIME
  use arg traceObj
  return "(#" adjRight(traceObj~nr,4)"," traceObj~timeStamp~longTime")" traceObj~makeString

