/* a program that will run the supplied Rexx program and saves the trace in

   - three human legible JSON-files with option='N', 'S', 'F' and 'P'
   - three CSV-files with option='N', 'S', 'F' and 'P'

   Note: all three JSON- and three CSV-files should be identical as all TraceObject
         entries will be written to the file
*/


parse arg fn
if \SysFileExists(fn) then
do
   say "file ["fn"] does not exist, aborting ..."
   exit -1
end
lastDotPos=fn~lastPos('.')
if lastDotPos=0 then
   fnOut=fn"_out_"
else
   fnOut=left(fn,max(lastDotPos-1,1)) || "_"

do counter c option over "N", "T", "S", "F", "P"   -- Normal, Thread, Standard, Full, Profiling
   say "RUN #" c": option='"option"' ..." "(so far there were" .traceObject~counter "TraceObjects created)"
   arr=.array~new
   .TraceObject~option=option       -- global
   .TraceObject~collector=arr       -- global
   call runner fn, option, arr
say "... RETURNED from CALL" fn "...." "returned in RUN #" c "(this run created another" arr~items "TraceObjects)"

   say "-"~copies(79)
   say
end

--- create the json and csv files

   tmpFnOut=fnOut || option

----------
   tmpJsonFn=tmpFnOut"_M.json"
say "... creating JSON-file ["tmpJsonFn"] (Minimal)"
   call toJsonFile tmpJsonFn, arr, 'M'

   tmpJsonFn=tmpFnOut"_C.json"
say "... creating JSON-file ["tmpJsonFn"] (CRLF (somewhat legible)"
   call toJsonFile tmpJsonFn, arr, 'C'

   tmpJsonFn=tmpFnOut"_H.json"
say "... creating JSON-file ["tmpJsonFn"] (Human legible)"
   call toJsonFile tmpJsonFn, arr, 'H'
----------
   tmpCsvFn=fnOut"b_no_Title.csv"
say "... creating CSV-file ["tmpCsvFn"] (no title line)"
   call toCsvFile tmpCsvFn, arr, .false

   tmpCsvFn=fnOut"a_with_Title.csv"
say "... creating CSV-file ["tmpCsvFn"] (with title line)"
   call toCsvFile tmpCsvFn, arr, .true
----------



::routine writeFile
  use strict arg fn, data
say .context~name":" "fn=["fn"]"
  s=.stream~new(fn)
  s~~open("write replace") ~~charout(data) ~~close -- write all bytes
  return

::routine readFile
  use strict arg fn
say .context~name":" "fn=["fn"]"
  s=.stream~new(fn)
  s~~open("read shared")
  data=s~charIn(1,s~chars)    -- read all bytes
  s~close
  return data


::routine runner     -- in case the called program has an error, intercept it here
   use arg fn, option, arr
   signal on syntax
   say
   say "about to call ["fn"] with option='"option"' arr~items="arr~items "..."
   call (fn)

syntax:
say "#" .line "..."
   if VAR("SIGL") then  -- a condition occurred?
   do
      co=condition('o')
      say "SIGNAL   from calling ["fn"] with option="option "arr~items="arr~items "..."
      say "        name:" co~name":" co~instruction "sigl:" sigl
      say "   errortext:" co~errortext
      say "     message:" co~message
      say
   end
   else
      say "returned from calling ["fn"] with option='"option"' arr~items="arr~items "..."
   say ".TraceObject~option:" .TraceObject~option ".TraceObject~collector:" .TraceObject~collector~objectname
   say
   say "waiting a little bit to have all threads exited (they generate additional traceObjects) ..."
   call syssleep .1
   .TraceObject~collector=.nil      -- now stop collecting
   say center(" option '"option"': arr~items:" arr~items "",69,"-")


::requires "traceutil.cls"    -- get TraceObject related utilities
