/* a program that will read in the JSON and CSV files that start with the supplied file name
   and creates a TraceObject array of it which will be shown with .TraceObject~option set
   to 'N', 'S' and 'F'.

*/


parse arg fnBegins option
if option="" then option="N"
if pos(option~upper,"NTSF")=0 then
   raise syntax 93.914 array (1, '"N" (ormal), "S" (standard) or "F" (full)', arg(1))

.traceObject~option=option    -- set option for makeString formatting

fileList=.array~new
if SysFileExists(fnBegins) then
   fileList~append(fnBegins)
else  -- search and collect
do ext over ".csv", ".json"
   if fnBegins~endsWith('*') then fnPattern=fnBegins || ext
                             else fnPattern=fnBegins || '*' || ext
say .line": fnPatter=["fnPattern"]"
   call sysFileTree fnPattern, "files.", "fo"
   do i=1 to files.0
      fileList~append(files.i)
   end
end

if fileList~items=0 then
do
   say 'no csv or json files found that start with ['fnBegins'], aborting ...'
   exit -1
end

   tab="09"x
do counter c fn over fileList
   say adjRight(c,3)":" fn
   say
   if fn~endsWith(".json") then
      arr=fromJsonFile(fn)    -- returns array of TraceObjects
   else
   do
      arr=fromCsvFile(fn)     -- returns array of TraceObjects
   end
   say
   info=" round #" c "returned" arr~items "from" fn ""
   say info
   say
   say tab "displaying all the re-established traceObjects with option '"option"'"
   say
   do counter c2 traceObj over arr
      -- say traceObj "/" traceObj~class "/" traceObj~allIndexes~sort~toString('L',',')
      say traceObj   -- makeString will format it according to .TraceObject~option
   end
   say info~center(79,'-')
end

say "... just testing datetime reconversion..."
traceObj=arr[1]
dt=traceObj~timestamp
say "dt~class:" dt~class "dt:" dt


::requires "traceutil.cls"    -- get TraceObject related utilities
